;**************************************************************************
;*                                                                        *
;* Program     : C2004A2                                                  *
;* Title       : 20x4 LCD Module Driving Program                          *
;*                                                                        *
;* MCU         : AT89C51, 12MHz                                           *
;* Assembler   : AD2500                                                   *
;* Programmer  : Stephen Yu, 25 Oct. 2002                                 *
;* File name   : C2004A2.asm                                              *
;* Eval. Board : LKEB-C01-A1                                              *
;* Data format : 8-bit parallel                                           *
;* Description : Drive 20 characters x 4 lines LCD module with KS0066     *
;*               controller or equivalent                                 *
;* LCD Module  : LKC-2004-A2 or equivalent                                *
;* Copyright   : Lcdmod Kit                                               *
;* URL         : http://www.lcdmodkit.com                                 *
;**************************************************************************

INITIAL:
;**************************************************************************
ENABLE  REG     P3.3            ; Assign LCD E pin = P3.3
R_W     REG     P3.4            ; Assign LCD R/W pin = P3.4
RS      REG     P3.5            ; Assign LCD RS pin = P3.5
CLICK   REG     P3.2            ; Push button input = P3.2
                                ; Port 1 as data bus
;**************************************************************************

COM     EQU     20H
DAT     EQU     21H

        ORG     00H             ; Code address

        CALL    RESET           ; Reset the LCD Module
        CALL    INT             ; Intialize the LCD Module

MAIN:                        ; ( MAIN PROGRAM )
        LCALL   MSG          ; Display message I
        LCALL   BUTTON       ; Wait on clicking the push button
        LCALL   CLEAR        ; Clear the LCD Module

        LCALL   ADV          ; Display message II
        LCALL   BUTTON

        LCALL   SCROLL       ; Message II scrolling
        LCALL   BUTTON
        LCALL   CLEAR

        LCALL   SHOW_CG      ; Display the font in CG RAM
        LCALL   BUTTON
        LCALL   CLEAR

        JMP     MAIN


INT:                         ; ( PROCEDURE FOR INITIALIZE THE LCD MODULE )
        MOV     COM,#00111000B  ; Function set, 8-bit operation & dual line
        CALL    CW

        MOV     COM,#00001100B  ; Display on, (D = 0, C = 1, B = 0)
        CALL    CW

        MOV     COM,#00000110B  ; Set entry mode, Cursor shift right
        CALL    CW

        MOV     COM,#00000010B  ; Display cursor return home
        CALL    CW

        MOV     COM,#00000001B  ; Clear display
        CALL    CW
        RET

RESET:                       ; ( PROCEDURE FOR RESET THE LCD MODULE )
        CLR     ENABLE          ; Set E = 0, disable LCD
        MOV     R2,#250         ; Delay 62.5ms
RESET1: MOV     R1,#250
        DJNZ    R1,$
        DJNZ    R2,RESET1
        MOV     A,#00110000B    ; Function set
        CALL    CW
        MOV     R2,#50          ; Delay parameter
RESET2: MOV     R1,#100
        DJNZ    R1,$            ; about 50*100*1 us = 5 ms
        DJNZ    R2,RESET2
        MOV     A,#00110000B    ; Function set
        CALL    CW
        MOV     R1,#100         ; Delay parameter
        DJNZ    R1,$            ; about 100*1 us = 100 us
        MOV     A,#00110000B    ; Function set
        CALL    CW
        RET

HOME:                        ; ( PROCEDURE FOR RETURN HOME )
        MOV     COM,#02H
        LCALL   CW
        RET

CLEAR:                       ; ( PROCEDURE FOR CLEAR THE LCD MODULE )
        MOV     COM,#00000001B
        CALL    CW
        RET

MSG:                         ; ( PROCEDURE FOR DISPLAYING MESSAGE I )
        MOV     COM,#10000000B  ; Display Data RAM address = 00H
        CALL    CW
        MOV     R3,#40           ; 40 characters on first line
        MOV     DPTR,#FRONT1
MSG1:   MOV     A,#0
        MOVC    A,@A+DPTR
        MOV     DAT,A
        CALL    DW
        INC     DPTR
        DJNZ    R3,MSG1         ; Check whether at the end of the 1st line?

                                ; (LCD 2nd line) display Data RAM address
        MOV     COM,#11000000B  ; Display Data RAM address = 40H
        CALL    CW
        MOV     R3,#40           ; 40 characters on second line
        MOV     DPTR,#FRONT2
MSG2:   MOV     A,#0
        MOVC    A,@A+DPTR
        MOV     DAT,A
        CALL    DW
        INC     DPTR
        DJNZ    R3,MSG2         ; Check whether at the end of the 2nd line?
        RET

ADV:                         ; ( PROCEDRUE FOR DISPLAYING MESSAGE II )
        MOV     COM,#10000000B  ; Display Data RAM address = 00H
        CALL    CW
        MOV     R3,#19          ; 19 characters on first line
        MOV     DPTR,#BANNER
ADV1:   MOV     A,#0
        MOVC    A,@A+DPTR
        MOV     DAT,A
        CALL    DW
        INC     DPTR
        DJNZ    R3,ADV1         ; Check whether at the end ?
        RET

SCROLL:                      ; ( PROCEDURE FOR SCREEN SCROLLING )
       MOV      COM,#80H
       LCALL    CW
       MOV      R2,#30
SCROLL1:
        LCALL   DELAY
        MOV     COM,#18H        ; SHIFT LEFT
        LCALL   CW
        DJNZ    R2,SCROLL1

        MOV     COM,#80H
        LCALL   CW
        MOV     R2,#30
SCROLL2:
        LCALL   DELAY
        MOV     COM,#1CH        ; SHIFT RIGHT
        LCALL   CW
        DJNZ    R2,SCROLL2
        RET

SHOW_CG:                     ; ( PROCEDURE FOR DISPLAY THE FONT OF CG RAM )
       LCALL    CG_RAM
       MOV      COM,#80H
       LCALL    CW
       MOV      DPTR,#LOGO
       MOV      R2,#8
       MOV      R3,#00H
SHOW_CG1:
       MOV      A,R3
       MOVC     A,@A+DPTR
       MOV      DAT,A
       LCALL    DW
       INC      R3
       DJNZ     R2,SHOW_CG1
       RET


CG_RAM:                     ; ( PROCEDURE FOR MAKING FONT IN CG RAM )
       MOV      COM,#40H
       LCALL    CW
       MOV      R2,#8
       MOV      R3,#00H
       MOV      DPTR,#RAM_DATA
CG_RAM1:
        MOV     A,R3
        MOVC    A,@A+DPTR
        MOV     DAT,A
        LCALL   DW
        INC     R3
        DJNZ    R2,CG_RAM1
        RET

CW:                          ; ( PROCEDURE FOR COMMAND WRITE )
        CALL    CHECK_BUSY
        MOV     P1,COM
        CLR     RS
        CLR     R_W
        SETB    ENABLE
        CLR     ENABLE
        RET

DW:                          ; ( PROCEDURE FOR DATA WRITE )
        CALL    CHECK_BUSY
        MOV     P1,DAT
        SETB    RS
        CLR     R_W
        SETB    ENABLE
        CLR     ENABLE
        RET

CHECK_BUSY:                  ; ( PROCEDURE FOR CHECK BUSY OF LCD CONTROLLER )
        MOV     P1,#0FFH
        CLR     RS
        SETB    R_W
        SETB    ENABLE
        MOV     C,P1.7
        CLR     ENABLE
        JC      CHECK_BUSY
        RET

BUTTON:                      ; ( PROCEDURE FOR DETECT CLICKING OF PUSH BUTTON )
        JB      CLICK,$
        ACALL   WAIT         ; ESCAPE FROM THE DEBOUNCING OF BUTTON
        JNB     CLICK,$
        ACALL   WAIT
        RET


DELAY:                       ; ( PROCEDURE FOR TIME DELAY )
      PUSH      5
      PUSH      6
      PUSH      7
      MOV       R5,#250
DL1:  MOV       R6,#20
DL2:  MOV       R7,#20
DL3:  DJNZ      R7,DL3
      DJNZ      R6,DL2
      DJNZ      R5,DL1
      POP       7
      POP       6
      POP       5
      RET

WAIT:                        ; ( PROCEDURE FOR TIME WAITING )
        PUSH    6
        PUSH    7
        MOV     R6,#100
WAIT2:  MOV     R7,#200
        DJNZ    R7,$
        DJNZ    R6,WAIT2
        POP     7
        POP     6
        RET

FRONT1  DB      "Lcdmod Kit is an on-Also we provide a tu"
FRONT2  DB      "line retailer of LCDrnkey solution.     "

BANNER  DB      " www.lcdmodkit.com "

LOGO    DB      00H,00H,00H,00H,00H,00H,00H,00H;

RAM_DATA
        DB      00H,11H,1BH,15H,11H,11H,0EH,00H;
        END
